import collections


class Styles(object):
    @staticmethod
    def create(paragraph_styles=None, character_styles=None, table_styles=None, numbering_styles=None):
        if paragraph_styles is None:
            paragraph_styles = {}
        if character_styles is None:
            character_styles = {}
        if table_styles is None:
            table_styles = {}
        if numbering_styles is None:
            numbering_styles = {}

        return Styles(
            paragraph_styles=paragraph_styles,
            character_styles=character_styles,
            table_styles=table_styles,
            numbering_styles=numbering_styles,
        )

    def __init__(self, paragraph_styles, character_styles, table_styles, numbering_styles):
        self._paragraph_styles = paragraph_styles
        self._character_styles = character_styles
        self._table_styles = table_styles
        self._numbering_styles = numbering_styles

    def find_paragraph_style_by_id(self, style_id):
        return self._paragraph_styles.get(style_id)

    def find_character_style_by_id(self, style_id):
        return self._character_styles.get(style_id)

    def find_table_style_by_id(self, style_id):
        return self._table_styles.get(style_id)

    def find_numbering_style_by_id(self, style_id):
        return self._numbering_styles.get(style_id)


Styles.EMPTY = Styles(
    paragraph_styles={},
    character_styles={},
    table_styles={},
    numbering_styles={},
)


def read_styles_xml_element(element):
    paragraph_styles = {}
    character_styles = {}
    table_styles = {}
    numbering_styles = {}
    styles = {
        "paragraph": paragraph_styles,
        "character": character_styles,
        "table": table_styles,
    }

    for style_element in element.find_children("w:style"):
        style = _read_style_element(style_element)
        element_type = style_element.attributes["w:type"]
        if element_type == "numbering":
            numbering_styles[style.style_id] = _read_numbering_style_element(style_element)
        else:
            style_set = styles.get(element_type)
            if style_set is not None:
                style_set[style.style_id] = style

    return Styles(
        paragraph_styles=paragraph_styles,
        character_styles=character_styles,
        table_styles=table_styles,
        numbering_styles=numbering_styles,
    )


Style = collections.namedtuple("Style", ["style_id", "name"])


def _read_style_element(element):
    style_id = element.attributes["w:styleId"]
    name = element.find_child_or_null("w:name").attributes.get("w:val")
    return Style(style_id=style_id, name=name)


NumberingStyle = collections.namedtuple("NumberingStyle", ["num_id"])


def _read_numbering_style_element(element):
    num_id = element \
        .find_child_or_null("w:pPr") \
        .find_child_or_null("w:numPr") \
        .find_child_or_null("w:numId") \
        .attributes.get("w:val")

    return NumberingStyle(num_id=num_id)



import collections


class Relationships(object):
    def __init__(self, relationships):
        self._targets_by_id = dict(
            (relationship.relationship_id, relationship.target)
            for relationship in relationships
        )
        self._targets_by_type = collections.defaultdict(list)
        for relationship in relationships:
            self._targets_by_type[relationship.type].append(relationship.target)
    
    def find_target_by_relationship_id(self, key):
        return self._targets_by_id[key]
    
    def find_targets_by_type(self, relationship_type):
        return self._targets_by_type[relationship_type]


Relationships.EMPTY = Relationships([])


Relationship = collections.namedtuple("Relationship", ["relationship_id", "target", "type"])


def read_relationships_xml_element(element):
    children = element.find_children("relationships:Relationship")
    return Relationships(list(map(_read_relationship, children)))


def _read_relationship(element):
    relationship = Relationship(
        relationship_id=element.attributes["Id"],
        target=element.attributes["Target"],
        type=element.attributes["Type"],
    )
    return relationship



#!/usr/bin/env python

import os
import sys
from setuptools import setup

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


setup(
    name='mammoth',
    version='1.4.12',
    description='Convert Word documents from docx to simple and clean HTML and Markdown',
    long_description=read("README"),
    author='Michael Williamson',
    author_email='mike@zwobble.org',
    url='http://github.com/mwilliamson/python-mammoth',
    packages=['mammoth', 'mammoth.docx', 'mammoth.html', 'mammoth.styles', 'mammoth.styles.parser', 'mammoth.writers'],
    entry_points={
        "console_scripts": [
            "mammoth=mammoth.cli:main"
        ]
    },
    keywords="docx word office clean html markdown md",
    install_requires=[
        "cobble>=0.1.3,<0.2",
    ],
    python_requires='>=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*',
    license="BSD-2-Clause",
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
)

